# Python imports

# Lib imports
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

# Application imports
from libs.dto.lsp_messages import get_message_obj
from core.controllers.lsp_controller import LSPController
from .buttons.top_button_box import TopButtonBox
from .enteries.lsp_message_source_view import LspMessageSourceView
from .buttons.bottom_button_box import BottomButtonBox



class LSPMessageBox(Gtk.Box):
    def __init__(self):
        super(LSPMessageBox, self).__init__()

        self._setup_styling()
        self._setup_signals()
        self._subscribe_to_events()
        self._load_widgets()

        self.show()


    def _setup_styling(self):
        ctx = self.get_style_context()
        ctx.add_class("lsp-message-box")

        self.set_orientation( Gtk.Orientation.VERTICAL )
        self.set_hexpand(True)
        self.set_vexpand(True)

    def _setup_signals(self):
        ...

    def _subscribe_to_events(self):
        ...

    def _load_widgets(self):
        scrolled_win        = Gtk.ScrolledWindow()
        self.lsp_msg_src_vw = LspMessageSourceView()
        self.top_buttons    = TopButtonBox()
        bottom_buttons      = BottomButtonBox()

        scrolled_win.show_all()

        scrolled_win.add(self.lsp_msg_src_vw)
        self.add(self.top_buttons)
        self.add(scrolled_win)
        self.add(bottom_buttons)

    def create_lsp_controller(self, language, log_list):
        self.lsp_controller = LSPController()
        self.lsp_controller.set_language(language)
        self.lsp_controller.set_log_list(log_list)


    def update_message_id_label(self):
        self.top_buttons.update_message_id_lbl(
            self.lsp_controller.get_message_id()
        )

    def send_initialize_message(self):
        parent         = self.get_parent()
        init_ops       = parent.init_ops_src_vw.get_text_str()
        workspace_file = self.top_buttons.get_workspace_path()
        workspace_uri  = self.top_buttons.get_workspace_uri()

        self.lsp_controller.send_initialize_message(init_ops, workspace_file, workspace_uri)
        self.update_message_id_label()

    def send_initialized_message(self):
        self.lsp_controller.send_initialized_message()

    def button_send_notification(self):
        message = get_message_obj( self.lsp_msg_src_vw.get_text_str() )
        self.lsp_controller.send_notification(message["method"], message["params"])

    def button_send_request(self):
        message = get_message_obj( self.lsp_msg_src_vw.get_text_str() )
        self.lsp_controller.send_request(message["method"], message["params"])
        self.update_message_id_label()

    def start_stop_lsp(self):
        parent        = self.get_parent()
        _command: str = parent.alt_command_entry.get_text()
        # _command: str = parent.command_entry.get_text()
        # _command: str = parent.socket_entry.get_text()
        command: []   = _command.split() if len( _command.split() ) > 0 else [ _command ]

        self.lsp_controller.set_start_command(command)
        self.lsp_controller.start_stop_lsp()
        self.lsp_controller.unset_start_command()
        self.update_message_id_label()